#!/usr/bin/perl -w
use strict;
require File::Spec;


my($pgp,$buffered_io,$recipientKey,$senderKey,$recipientPassphrase,$senderPassphrase,$pgpOptions,$debug,$testfolder,$keyring,$file1,$file2,$file3,$armor,$compress);
sub main();

use constant TRUE => 1;
use constant FALSE => 0;

# the full path to the pgp commandline executable
$pgp = '"C:\Program Files (x86)\PGP Corporation\PGP Command Line\pgp.exe"';

# should buffered IO be explicitly turned on? (see PGP Command Line User's Guide section 'Standard Input, Output, and Error')
$buffered_io=FALSE;

# should we use ASCII-armored output or plain binary data?
$armor = TRUE;

# should we use compression before encryption?
$compress = TRUE;

# !!! consider not changing anything below this line
# filename of key to encrypt to
$recipientKey='STDIORecipientKey.asc';

# Passphrase of recipient key
$recipientPassphrase='password123';

# filename of key to sign with
$senderKey='STDIOTestKey.asc';

# Passphrase of signing key
$senderPassphrase='password';

# !! really don't change anything below this line !!
# file 1
$file1 = 'file1.txt';

# file 2
$file2 = 'file2.txt';

# file 3
$file3 = 'file3.txt';

# test folder name for output files
$testfolder = "test_out";

# keyring filename
$keyring = File::Spec->catfile(File::Spec->curdir(),File::Spec->catfile($testfolder,'test_keyring'));

# additional commandline options
$pgpOptions="--public-keyring $keyring.pkr --private-keyring $keyring.skr";

# turn on debugging
$debug = FALSE;

# call the main routine
main();

sub addEncryptCommandOptions($){
    # add aditional encrypt options to the commandline call according to the settings
    my $command = shift ();
    $command .= $pgpOptions." ";
    $command .= "--buffered-stdio " if $buffered_io;
    $command .= "--armor " if $armor;
    $command .= "--compress " if $compress;
    return $command;
}

sub addDecryptCommandOptions($){
    # add aditional decrypt options to the commandline call according to the settings
    my $command = shift ();
    $command .= $pgpOptions." ";
    $command .= "--buffered-stdio " if $buffered_io;
    return $command;
}

sub importKey($){
    my $filename = shift();
    my ($keyid,$exitcode,@return);
    # call pgp command line and capture bot the STDOUT and STDERR to $return variable
    my $command = "$pgp $pgpOptions --import $filename 2>&1";
    print "Executing: `$command` \n";
    @return = `$command`;
    $exitcode= $?;
    print " return value: $exitcode\n" if $debug;
    if ($exitcode != 0){
        die "Could not import key. Error: '$exitcode' Output: \n @return\n";
    }
    # check the return array (line by line output for the keyid that was imported)
    # the line the is written by pgp command line looks like this: 'STDIORecipientKey.asc:import key (0:key imported as 0x3E36701E STDIO Recipient Key)'
    foreach (@return) {
        print " checking line: '".$_."'\n" if $debug;
        # regular expression to match a keyid: (0x[0-9|A-F]{8})
        $keyid = $1 if ($_ =~ /.*key.imported.as.(0x[0-9|A-F]{8}).*/);
        print " matched keyid: '$keyid'\n" if $debug;
        return $keyid;
    }
    die "Couldn't get keyid from imported key\n";
}

sub encryptFile1($$){
    # encrypt + sign file1 using stdio for output
    my $senderKeyID = shift ();
    my $recipientKeyID = shift ();
    my ($command,$exitcode,$return,$output_filename);
    
    $output_filename = File::Spec->catfile(File::Spec->curdir(),File::Spec->catfile($testfolder,"$file1".".pgp"));
    
    $command = addEncryptCommandOptions("$pgp --encrypt $file1 --sign --signer $senderKeyID --passphrase \"$senderPassphrase\" --recipient $recipientKeyID -o - -q ");
    print "Executing: `$command` \n";
    $return = `$command`;
    $exitcode= $?;
    print " return value: $exitcode\n" if $debug;
    if ($exitcode != 0){
        die "pgp command line did not execute correctly. Error: '$exitcode' Output: \n '$return'\n";
    }
    
    # the encrypted message is now in $return. write it to a file
    open(OUTFILE,">$output_filename") or die ("couldn't open output file\n");
    print OUTFILE $return;
    close (OUTFILE);
    
    return TRUE;
}

sub encryptFile2($$){
    # encrypt + sign file2 using stdio for input
    my $senderKeyID = shift ();
    my $recipientKeyID = shift ();
    my ($command,$exitcode,$return,@inData,$output_filename);
    
    $output_filename = File::Spec->catfile(File::Spec->curdir(),File::Spec->catfile($testfolder,"$file2".".pgp"));
    
    # here we first need to load the input file into a buffer
    # this reads the file line-by-line into an array
    open(INFILE,"<$file2") or die ("couldn't open input file\n");
    @inData = <INFILE>;
    close (INFILE);
    
    $command = addEncryptCommandOptions("$pgp --encrypt -i - -o $output_filename --overwrite remove --sign --signer $senderKeyID --passphrase \"$senderPassphrase\" --recipient $recipientKeyID -q ");
    print "Piping data into command: `$command` \n";
    # this is the perl way to pipe data to a command:
    open(COMMAND,"| $command") or die ("couldn't open process to pipe input to\n");
    # write out the data line-by-line
    foreach (@inData){
        print COMMAND $_; # write the current line to STDIN of the process
    }
    close (COMMAND); # close the process
    
    
    $exitcode= $?;
    print " return value: $exitcode\n" if $debug;
    if ($exitcode != 0){
        die "pgp command line did not execute correctly. Error: '$exitcode' Output: \n '$return'\n";
    }
   
    return TRUE;
}

sub decryptFile1($$){
    # decrypt + verify file1 using stdio for input
    my $senderKeyID = shift ();
    my $recipientKeyID = shift ();
    my ($command,$exitcode,$return,@inData,$output_filename,$infile);
    $infile = File::Spec->catfile(File::Spec->curdir(),File::Spec->catfile($testfolder,"$file1".".pgp"));
    $output_filename = File::Spec->catfile(File::Spec->curdir(),File::Spec->catfile($testfolder,"file1_decrypted.txt"));
    
    # here we first need to load the input file into a buffer
    # this reads the file line-by-line into an array
    open(INFILE,"<".$infile) or die ("couldn't open input file\n");
    @inData = <INFILE>;
    close (INFILE);
    
    # in this example we use the verification as separate process,
    # but this can also be captured from STDERR form commandline output when doing a decrypt operation
    
    $command = addDecryptCommandOptions("$pgp --verify -i - --passphrase \"$recipientPassphrase\" ");
    print "Piping data into command: `$command` \n";
    # this is the perl way to pipe data to a command:
    open(COMMAND,"| $command") or die ("couldn't open process to pipe input to\n");
    # write out the data line-by-line
    foreach (@inData){
        print COMMAND $_; # write the current line to STDIN of the process
    }
    close (COMMAND); # close the process
    
    $exitcode= $?;
    print " return value: $exitcode\n" if $debug;
    if ($exitcode != 0){
        print "Signature verification failed. Error: '$exitcode'\n";
    }else {
        print "Signature successfully verified. Error: '$exitcode'\n";
    }
   
    # now do the decryption
    
    $command = addDecryptCommandOptions("$pgp --decrypt -i - -o $output_filename --annotate --overwrite remove --passphrase \"$recipientPassphrase\" -q ");
    print "Piping data into command: `$command` \n";
    # this is the perl way to pipe data to a command:
    open(COMMAND,"| $command") or die ("couldn't open process to pipe input to\n");
    # write out the data line-by-line
    foreach (@inData){
        print COMMAND $_; # write the current line to STDIN of the process
    }
    close (COMMAND); # close the process
    
    
    $exitcode= $?;
    print " return value: $exitcode\n" if $debug;
    if ($exitcode != 0){
        die "pgp command line did not execute correctly. Error: '$exitcode' Output: \n '$return'\n";
    }
   
    return TRUE;
}

sub decryptFile2($$){
    # decrypt + verify file2 using stdio for output
    my $senderKeyID = shift ();
    my $recipientKeyID = shift ();
    my ($command,$exitcode,$return,$outVerify,@outData,$output_filename,$verified,$sig_date,$sig_key,$sig_algo,$infile);
    $infile = File::Spec->catfile(File::Spec->curdir(),File::Spec->catfile($testfolder,"$file2".".pgp"));
    $output_filename = File::Spec->catfile(File::Spec->curdir(),File::Spec->catfile($testfolder,"file2_decrypted.txt"));
    $verified=FALSE;
    
    # in this example we use the verification as separate process,
    # but this can also be captured from STDERR form commandline output when doing a decrypt operation
    $command = addDecryptCommandOptions("$pgp --verify -i $infile --passphrase \"$recipientPassphrase\" 2>&1");
    
    print "Executing: `$command` \n";
    $outVerify = `$command`;
    $exitcode= $?;
    print " return value: $exitcode\n" if $debug;
    if ($exitcode != 0){
        die "pgp command line did not execute correctly. Error: '$exitcode' Output: \n '$return'\n";
    }
    
    # the output of STDIO & STDERR is now in @outData. check if verification succeeded
    foreach (split("\n",$outVerify)){
        print " checking line: '".$_."'\n" if $debug;
        # regular expression to check for message: stdin:verify (3035:good signature)
        $verified = TRUE if ($_ =~ /.*3035:good.signature.*/);
        $sig_date = $1 if ($_ =~ /.*3040:signature.created.(.*)\).*/);
        $sig_key = $1 if ($_ =~ /.*3177:message.signed.by.key.ID.(0x[0-9|A-F]{8})*/);
        $sig_algo = $1 if ($_ =~ /.*3170:signature.hash.(.*)\).*/);
    }
    
    if ($verified){
        print "signature successfully verified\n";
        print " signature date: $sig_date\n";
        print " signing key: $sig_key\n";
        print " signature hash algorithm: $sig_algo\n";
    }else {
        print "signature verification failed\n";
    }
    
    # now do the decryption
    $command = addDecryptCommandOptions("$pgp -q --decrypt -o - -i $infile --passphrase \"$recipientPassphrase\" ");
    
    print "Reading data from pipe: `$command` \n";
    # using a slightly different method here
    open (COMMAND,"$command |");
    @outData = <COMMAND>;
    close (COMMAND);
    $exitcode= $?;
    print " return value: $exitcode\n" if $debug;
    if ($exitcode != 0){
        die "pgp command line did not execute correctly. Error: '$exitcode' Output: \n '$return'\n";
    }
    
    # the decrypted message is now in $return. write it to a file
    open(OUTFILE,">$output_filename") or die ("couldn't open output file\n");
    foreach (@outData) {
        print OUTFILE $_;
    }
    close (OUTFILE);
    
    return TRUE;
}

sub main(){
    # this sub runs through several usecases
    my ($senderKeyID,$recipientKeyID,$result,$command);
    
    print 'Starting STDIO demo of PGP Command Line (c) PGP Corporation, azengel@pgp.com'."\n";
    
    # create test output folder
    mkdir($testfolder) if (!-d $testfolder);
    
    # create keyrings
    $result = `$pgp -q --create-keyrings --public-keyring $keyring.pkr --private-keyring $keyring.skr`;
    die ("Couldn't create keyrings") if ($? != 0);
    
    # import the recipient key
    $senderKeyID=importKey($senderKey);
    
    # import the senders key
    $recipientKeyID=importKey($recipientKey);
    
    # encrypt + sign file1 using stdio for output
    $result=encryptFile1($senderKeyID,$recipientKeyID);
    
    # encrypt + sign file2 using stdio for input
    $result=encryptFile2($senderKeyID,$recipientKeyID);
    
    # decrypt + verify file1 using stdio for input
    $result=decryptFile1($senderKeyID,$recipientKeyID);
    
    # decrypt + verify file2 using stdio for output
    $result=decryptFile2($senderKeyID,$recipientKeyID);
    
    print "Finished STDIO demo of PGP Command Line, see folder '$testfolder' for results\n";
}

